#!/bin/sh

set -e

both_apis_ver="eb5c9a8dd9eca5d1dd71962687f76d8d142fd1e2"

st="$(i2cget -y 1 0x2a 0x01 w)"

if [[ "${st:0:2}" != "0x" ]]; then
	echo "can't get status word" >&2
	exit 1
fi

if [[ "$((st & 0x3))" == "0" ]]; then
	mcu=stm32
elif [[ "$((st & 0x3))" == "1" ]]; then
	mcu=gd32
else
	echo "unknown mcu" >&2
fi

if [[ "$((st & 0x4))" == "4" ]]; then
	feat="$(i2cget -y 1 0x2a 0x10 w)"
	if [[ "${feat:0:2}" != "0x" ]]; then
		echo "can't get features" >&2
		exit 1
	fi
else
	feat=0x0
fi

if [[ "$((st & 0x8))" == "0" ]]; then
	suffix="-user-regulator"
else
	suffix=""
fi

if [[ "$((feat & 0x1))" == "1" ]]; then
	rev="rev32"
else
	rev="rev23"
fi

app_ver=$(i2ctransfer -y 1 w1@0x2a 0x0a r20 | sed -e 's/0x//g;s/ //g')
boot_ver=$(i2ctransfer -y 1 w1@0x2a 0x0e r20 | sed -e 's/0x//g;s/ //g')

if [[ "${#app_ver}" != 40 ]]; then
	echo "can't get app version" >&2
	exit 1
fi

if [[ "${#boot_ver}" != 40 ]]; then
	echo "can't get boot version" >&2
	exit 1
fi

again=0

if [[ "$((feat & 0x20))" == "32" || "${app_ver}" == "${both_apis_ver}" ]]; then
	src=newest
else
	src=both_apis
	again=1
fi

prefix="${mcu}-${rev}${suffix}"

if [[ "$((feat & 0x100))" == "256" ]]; then
	./mcutool -f "${src}/${prefix}.boot.bin" -B
	./mcutool -f "${src}/${prefix}.app.bin"
else
	./mcutool -f "${src}/${prefix}.app.bin"
	again=1
fi

if [[ "${again}" == "1" ]]; then
	echo
	echo "Please reboot and run again"
	echo
fi

exit 0
